// ############################################################################################
//  Fractal.cc
//  A demo application for VortexGE
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

//#define __HIGH_RESOLUTION__

// Common headers for demo application
#include "DemoCommon.h"


// Just for shortcut
typedef CBasicApplication<CEventHandler> MyApplicationBase;

// Specialized basic application class
class MyApplication : public MyApplicationBase {
  public:
    MyApplication();
    virtual void EventKeyPress(EKeySymbol eKeySym);
    virtual void EventUpdateFrame(bool bWindowIsActive);
    virtual void EventCloseWindow();

  private:
    CImageBuilder csIBuild;
    CImage        csImg[6];
    uint8_t       un8Seq;
};


// Main program
int main()
{
  MyApplication csMyApp;
  return(csMyApp.DoMainLoop());
}


// MyApplication implementation
MyApplication::MyApplication()
: MyApplicationBase("Demo : Fractal", NULL, 640, 480, 0, 0, EAIM_Disabled),
  un8Seq(0)
{
  // Tell the user to be patient
  ShowInitSystemPleaseWaitMessage(iVideoEnv());

  // Create an empty image
  csIBuild.CreateEmptyImage(640, 480);

  // Generate definite area fractals page 1
  VInfo(_NFL_, "Drawing definite area fractals :");
  {
    FractalMandelbrot csF1;
    FractalJulia      csF2;
    FractalDragon     csF3;
    FractalFExp       csF4;
    VPrint("  Mandelbrot\n");  csF1.Draw(&csIBuild,   0,   0, 318, 238);
    VPrint("  Julia\n");       csF2.Draw(&csIBuild, 321,   0, 639, 238);
    VPrint("  Dragon\n");      csF3.Draw(&csIBuild,   0, 241, 318, 479);
    VPrint("  FExp\n");        csF4.Draw(&csIBuild, 321, 241, 639, 479);
  }
  csIBuild.TransferToImage(csImg[0]);
  csIBuild.ClearImage();

  // Generate definite area fractals page 2
  {
    FractalFSin  csF1;
    FractalFCos  csF2;
    FractalFSinh csF3;
    FractalFCosh csF4;
    VPrint("  FSin\n");  csF1.Draw(&csIBuild,   0,   0, 318, 238);
    VPrint("  FCos\n");  csF2.Draw(&csIBuild, 321,   0, 639, 238);
    VPrint("  FSinh\n"); csF3.Draw(&csIBuild,   0, 241, 318, 479);
    VPrint("  FCosh\n"); csF4.Draw(&csIBuild, 321, 241, 639, 479);
  }
  csIBuild.TransferToImage(csImg[1]);
  csIBuild.ClearImage();

  // Generate definite area fractals page 3
  {
    FractalSierspinski csF1;
    FractalSun         csF2;
    FractalNebula      csF3;
    FractalCoral       csF4;
    VPrint("  Sierspinski\n"); csF1.Draw(&csIBuild,   0,   0, 318, 238);
    VPrint("  Sun\n");         csF2.Draw(&csIBuild, 321,   0, 639, 238);
    VPrint("  Nebula\n");      csF3.Draw(&csIBuild,   0, 241, 318, 479);
    VPrint("  Coral\n");       csF4.Draw(&csIBuild, 321, 241, 639, 479);
  }
  csIBuild.TransferToImage(csImg[2]);
  csIBuild.ClearImage();

  // Generate definite area fractals page 4
  {
    FractalTargetBox csF1;
    VPrint("  TargetBox\n"); csF1.Draw(&csIBuild,   0,   0, 318, 238);
  }
  csIBuild.TransferToImage(csImg[3]);
  csIBuild.ClearImage();

  // Generate indefinite area fractals page 5
  VPrint("Drawing indefinite area fractals :\n");
  {
    FractalFern      csF1;
    FractalBamboo    csF2;
    FractalMapleLeaf csF3;
    VPrint("  Fern1\n");      csF1.Draw(&csIBuild, 175, 230, 70, 600);
                              csF1.Draw(&csIBuild, 180, 230, 70,   0);
                              csF1.Draw(&csIBuild, 185, 230, 70,-600);
    VPrint("  Bamboo1\n");    csF2.Draw(&csIBuild, 510, 275);
    VPrint("  MapleLeaf1\n"); csF3.Draw(&csIBuild, 345, 410, 270);
  }
  csIBuild.TransferToImage(csImg[4]);
  csIBuild.ClearImage();

  // Generate indefinite area fractals page 6
  {
    FractalCirrus   csF1;
    FractalStargate csF2;
    VPrint("  Cirrus1\n");   csF1.Draw(&csIBuild, 140, 110, 80);
    VPrint("  Stargate1\n"); csF2.Draw(&csIBuild, 140, 310);
                             csF2.Draw(&csIBuild, 140, 310, 3);
  }
  csIBuild.TransferToImage(csImg[5]);

  // Free image
  csIBuild.FreeImage();
}

void MyApplication::EventKeyPress(EKeySymbol eKeySym)
{
  switch(eKeySym) {
    case VKS_ESC : EventCloseWindow(); break;
    case ' '     : un8Seq = (un8Seq < 5) ? (un8Seq + 1) : 0; break;
    default      : break;
  }
}

void MyApplication::EventUpdateFrame(bool bWindowIsActive)
{
  // Update frame (if the application is not active)
  if(!bWindowIsActive) {
    iVideoEnv().PutRenderImageOnDisplay();
    iVideoEnv().SetForegroundColor(iVideoEnv().CreateColor(255, 255, 255));
    iVideoEnv().UseDefaultBigFont();
    iVideoEnv().DrawText(0, 150, "Inactive State", ETA_CenterScreen);
    iVideoEnv().UseDefaultSmallFont();
    iVideoEnv().UpdateDisplay();
    return;
  }

  // Display the background image
  i3DEnv().ClearRenderBuffer(csImg[un8Seq]);

  // Update display
  iVideoEnv().PutRenderImageOnDisplay();
  iVideoEnv().SetForegroundColor(iVideoEnv().CreateColor(255, 255, 255));
  iVideoEnv().DrawText(10, 468, "Press SPACE to display the next fractals");
  iVideoEnv().DrawText(10, 20, "Press escape to exit ...");
  iVideoEnv().UpdateDisplay();
}

void MyApplication::EventCloseWindow()
{
  iVideoEnv().MessageBox("End Program", "Click \"OK\" to exit ...");
  MyApplicationBase::EventCloseWindow();
}
