// ############################################################################################
//  RotatingBox2.cc
//  A demo application for VortexGE
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

//#define __HIGH_RESOLUTION__

// Common headers for demo application
#include "DemoCommon.h"


// Just for shortcut
typedef CBasicApplication<> MyApplicationBase;

// Specialized basic application class
class MyApplication : public MyApplicationBase {
  public:
    MyApplication();
    virtual void EventKeyPress(EKeySymbol eKeySym);
    virtual void EventUpdateFrame(bool bWindowIsActive);
    virtual void EventCloseWindow();

  private:
    CPanel3DBuffer csGround, csBaseBox, csRotBox;
    CMatrix3D      csMatCamera;
    CImageBuilder  csImgBuild;
    CImage         csImgBack;
    CTexture       csTGround, csTBox;
    CHNode         csBaseNode, *pcsRotNode1, *pcsRotNode2;
    float          afCamPos[3], afCamTarget[3], fXDir;
    int16_t        n16XCnt;
};


// Main program
int main()
{
  MyApplication csMyApp;
  return(csMyApp.DoMainLoop());
}


// MyApplication implementation
MyApplication::MyApplication()
#ifdef __HIGH_RESOLUTION__
: MyApplicationBase("Demo : Rotating Box 2", NULL, 640, 480, 640, 480, EAIM_Disabled),
#else
: MyApplicationBase("Demo : Rotating Box 2", NULL, 640, 480, 320, 240, EAIM_Disabled),
#endif
  fXDir(0.2f), n16XCnt(200)
{
  // Tell the user to be patient
  ShowInitSystemPleaseWaitMessage(iVideoEnv());

  // Set wanted FPS
  SetWantedFPS(50.0f);

  // Load background image
  csImgBuild.LoadBMP("../media/image/back1.bmp", true);
  csImgBuild.ResizeToFitBackground();
  csImgBuild.AdjustBrightness(0.3, 0);
  csImgBuild.SmearEffect(5);
  csImgBuild.TransferToImage(csImgBack);

  // Load texture image 1
  try {
    csImgBuild.LoadJPG("../media/image/tex1.jpg", true);
    csImgBuild.Resize(256, 256);
    csImgBuild.SmearEffect(2);
  }
  catch(...) {
    csImgBuild.CreateNoiseImage(256, 256, 4, 22, 4, 2, 8, 2);
  }
  csImgBuild.TransferToTexture(csTGround);

  // Load texture image 2
  try {
    csImgBuild.LoadJPG("../media/image/tex2.jpg", true);
    csImgBuild.Resize(64, 64);

  }
  catch(...) {
    csImgBuild.CreateNoiseImage(64, 64, 18, 18, 20, 2, 2, 2);
    csImgBuild.OilPaintingEffect(1);
  }
  csImgBuild.TransferToTexture(csTBox);

  // Free image
  csImgBuild.FreeImage();

  // Create scene
  i3DEnv().SetFogState(8, 8, 12, 13);
  i3DEnv().SetDefaultOmniLightPosition(40, 200, 150);
  i3DEnv().SetDefaultOmniLightColor(31, 31, 16);
  i3DEnv().SetZClipping(150, 500);

  float    afVG[]    = { -100,-10,100,   -100,-10,200,   100,-10,200,   100,-10,100,
                         -100,  0,100,   -100,  0,200,   100,  0,200,   100,  0,100 };
  uint32_t aun32PG[] = { 3,2,1,0,   4,5,6,7,
                         0,4,7,3,   3,7,6,2,    2,6,5,1,   1,5,4,0 };
  csGround.InitFromMemory(afVG, 8, aun32PG, 6);
  csGround.SetTexture(&csTGround);
  csGround.CalcDefTexCoords();

  float    afVB[]    = { 0, 0,-20,   -20, 0,0,   0, 0,20,   20, 0,0,
                         0,35,-20,   -20,35,0,   0,35,20,   20,35,0 };
  uint32_t aun32PB[] = { 3,2,1,0,   4,5,6,7,
                         0,4,7,3,   3,7,6,2,   2,6,5,1,   1,5,4,0 };
  csBaseBox.InitFromMemory(afVB, 8, aun32PB, 6);
  csBaseBox.SetTexture(&csTBox);
  csBaseBox.CalcDefTexCoords();

  float    afVR[]    = { 0, 0,-10,   0, 0,10,   60, 0,10,   60, 0,-10,
                         0,10,-10,   0,10,10,   60,10,10,   60,10,-10 };
  uint32_t aun32PR[] = { 3,2,1,0,   4,5,6,7,
                         0,4,7,3,   3,7,6,2,   2,6,5, 1,   1,5,4,0 };
  csRotBox.InitFromMemory(afVR, 8, aun32PR, 6);
  csRotBox.SetTexture(&csTBox);
  csRotBox.CalcDefTexCoords();

  // Initialize camera
  //       * This is the same as in the "RotatingBox.cc"and is dumped just for
  //         showing that the result is similar with the next method
  csMatCamera.SetToIdentityMatrix();
  csMatCamera.Translate(0, -200, 100);
  csMatCamera.Rotate(-400, 0, 0);
  csMatCamera.DumpMatrix();
  //       * This one is using the new CMatrix3D::SetToLookAtMatrix()
  //       * That this method is easier but slower since SetToLookAtMatrix()
  //         performs complex calculations
  afCamPos[0]    = 0; afCamPos[1]    = 200; afCamPos[2]    = -100;
  afCamTarget[0] = 0; afCamTarget[1] =  24; afCamTarget[2] =  150;
  csMatCamera.SetToLookAtMatrix(afCamPos, afCamTarget);
  csMatCamera.DumpMatrix();

  // Initialize hierarchy
  #define ID_ROT1 2
  #define ID_ROT2 3
  try {
    csBaseNode.LoadVHN("../.mediaCaches/rotbox.vhn");
    pcsRotNode1 = csBaseNode.FindChild(ID_ROT1);
    pcsRotNode2 = csBaseNode.FindChild(ID_ROT2);
  }
  catch(...) {
    csBaseNode.SetID(1);
    csBaseNode.StcTranslate(0, -5, 150);
    pcsRotNode1 = csBaseNode.AddChild(ID_ROT1);
    pcsRotNode1->DynSetPosition(0, 20, 0);
    pcsRotNode2 = pcsRotNode1->AddChild(ID_ROT2);
    pcsRotNode2->StcTranslate(-30, 0, 0);
    pcsRotNode2->StcScale(0.5, 0.5, 0.5);
    pcsRotNode2->DynSetPosition(50, 10, 0);
    csBaseNode.SaveVHN("../.mediaCaches/rotbox.vhn");
  }
}

void MyApplication::EventKeyPress(EKeySymbol eKeySym)
{
  if(eKeySym == VKS_ESC)
    EventCloseWindow();
  else if(eKeySym == 'c') {
    afCamPos[0] = 0; afCamPos[1] = 200;
    csMatCamera.SetToLookAtMatrix(afCamPos, afCamTarget);
  }
  else if(eKeySym == 'a') {
    if(afCamPos[0] > float(-30.0)) {
      afCamPos[0] -= 10;
      csMatCamera.SetToLookAtMatrix(afCamPos, afCamTarget);
    }
  }
  else if(eKeySym == 's') {
    if(afCamPos[0] < float(30.0)) {
      afCamPos[0] += 10;
      csMatCamera.SetToLookAtMatrix(afCamPos, afCamTarget);
    }
  }
  else if(eKeySym == 'z') {
    if(afCamPos[1] > float(170.0)) {
      afCamPos[1] -= 10;
      csMatCamera.SetToLookAtMatrix(afCamPos, afCamTarget);
    }
  }
  else if(eKeySym == 'w') {
    if(afCamPos[1] < float(230.0)) {
      afCamPos[1] += 10;
      csMatCamera.SetToLookAtMatrix(afCamPos, afCamTarget);
    }
  }
}

void MyApplication::EventUpdateFrame(bool bWindowIsActive)
{
  // Update frame (if the application is not active)
  if(!bWindowIsActive) {
    iVideoEnv().PutRenderImageOnDisplay();
    iVideoEnv().SetForegroundColor(iVideoEnv().CreateColor(255, 255, 255));
    iVideoEnv().UseDefaultBigFont();
    iVideoEnv().DrawText(0, 150, "Inactive State", ETA_CenterScreen);
    iVideoEnv().UseDefaultSmallFont();
    iVideoEnv().UpdateDisplay();
    return;
  }

  // Update scene
  csBaseNode.DynTranslate(fXDir , 0, 0);
  pcsRotNode1->DynRotate(0, 40, 0);
  pcsRotNode2->DynRotate(0, -100, 0);
  csBaseNode.UpdateMatrix();
  n16XCnt++;
  if(n16XCnt > 400) {
    fXDir = -fXDir;
    n16XCnt = 0 ;
  }

  // Clear scene
  i3DEnv().ClearRenderBuffer(csImgBack);
  i3DEnv().ClearZBuffer();

  // Render the ground
  csGround.FogTGRender(csMatCamera);
  // Render the base box
  csBaseBox.PreDynGSRender(csBaseNode.Matrix());
  csBaseBox.DynFogTGRender(csMatCamera);
  // Render the 1st rotating box
  csRotBox.PreDynGSRender(pcsRotNode1->Matrix());
  csRotBox.DynFogTGRender(csMatCamera);
  // Render the 2nd rotating box
  csRotBox.PreDynGSRender(pcsRotNode2->Matrix());
  csRotBox.DynFogTGRender(csMatCamera);

  // Update display
  iVideoEnv().PutRenderImageOnDisplay();
  iVideoEnv().SetForegroundColor(iVideoEnv().CreateColor(255, 255, 255));
  iVideoEnv().DrawText(10, 20, GetFPSInfoString());
  iVideoEnv().DrawText(10, 50, "Press 'A' to move camera one step left");
  iVideoEnv().DrawText(10, 65, "Press 'S' to move camera one step right");
  iVideoEnv().DrawText(10, 80, "Press 'W' to move camera one step up");
  iVideoEnv().DrawText(10, 95, "Press 'Z' to move camera one step down");
  iVideoEnv().DrawText(10, 110, "Press 'C' to move camera back to center");
  iVideoEnv().DrawText(10, 140, "Press escape to exit ...");
  iVideoEnv().UpdateDisplay();
}

void MyApplication::EventCloseWindow()
{
  iVideoEnv().MessageBox("End Program", "Click \"OK\" to exit ...");
  MyApplicationBase::EventCloseWindow();
}
