// ############################################################################################
//  Font.cc
//  Implementation of CFontDesc class
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// VortexGE headers
#include "../System/SystemUtilities.h"
#include "Font.h"


namespace vge {

//
// CFontDesc implementation
//

// The default fonts
// If the fonts loading failed or the fonts looked uggly,
// try these sequentially/combinationally:
//   1. Try to change Weight::Regular to Weight::Any
//   2. Try to change Slant::Normal   to Slant::Any
//   3. Try to change the font sizes
//   4. Try to change the font families
//   5. Try to change 'iso8859'       to '*'
// Note that other fonts also can be choosen using xfontsel from console

const CFontDesc CFontDesc::DefaultSmallFont    ("*", "helvetica/times/courier/*", "12/10/11/9/8/7/13/14", Pitch::Any, Weight::Regular, Slant::Normal, Underline::None, "iso8859");
const CFontDesc CFontDesc::DefaultUtf8SmallFont("*", "*",                         "12/10/11/9/8/7/13/14", Pitch::Any, Weight::Regular, Slant::Normal, Underline::None, "*");

const CFontDesc CFontDesc::DefaultBigFont      ("*", "helvetica/times/courier/*", "24/23/22/21/20/19/18", Pitch::Any, Weight::Regular, Slant::Normal, Underline::None, "iso8859");
const CFontDesc CFontDesc::DefaultUtf8BigFont  ("*", "*",                         "24/23/22/21/20/19/18", Pitch::Any, Weight::Regular, Slant::Normal, Underline::None, "*");


void CFontDesc::ParseFamilyNames(vector<string> &fName) const
{ 
  VTokenize(fName, _family, "/"); 
}

void CFontDesc::ParsePointSizes(vector<string> &pSize) const
{ 
  VTokenize(pSize, _pointSize, "/"); 
}

} // namespace vge
