/** \file ExecutableByteCodes.h */
// ############################################################################################
//  ExecutableByteCodes.h
//  Definition & implementation of CLuaExecutable class
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This library is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################
#ifndef _VORTEX_LUA___EXECUTABLE_BYTE_CODES_H_
#define _VORTEX_LUA___EXECUTABLE_BYTE_CODES_H_

// VortexGE headers
#include "ScriptManager.h"


namespace vge {

  //!
  //! \brief A class used for storing the executable byte codes
  //!
  //! \api beta
  class VGE_EXPORT CLuaExecutable : VCLASS_UNCOPYABLE {
    public:
      //!
      //! This ctor allows user to specify a name for
      //! this executable instance.
      explicit CLuaExecutable(const char *nameOfThisExecutable = NULL);
      //!
      //! Standard dtor.
      ~CLuaExecutable() throw();

      // Access to the CLuaManager instance
      //!
      //! Direct access to the managing CLuaManager instance.
      //! \return NULL if this execution space is not managed.
      inline CLuaManager *ManagerInstance()
      { return(_mgr); }
      //!
      //! Direct access to the managing CLuaManager instance.
      //! \return NULL if this execution space is not managed.
      inline const CLuaManager *ManagerInstance() const
      { return(_mgr); }

      // ID functions
      //!
      //! Returns the name of this executable instance.
      inline const string &GetName() const
      { return(_nameID); }
      //!
      //! Set the name of this executable instance.
      inline void SetName(const char *newName)
      { _nameID = newName ? newName : ""; }

      // Direct access to the byte codes
      //!
      //! Returns the pointer to the byte codes buffer.
      inline const char *ByteCodes() const
      { return(&_byteCodes[0]); }
      //!
      //! Returns the size of the byte codes buffer.
      inline size_t ByteCodesSize() const
      { return(_byteCodes.size()); }

      // Script compilation helper functions
      //!
      //! Add (write) byte codes to this executable.
      bool Write(const char *bc, size_t size);
      //!
      //! Clear all byte codes in this excetuable.
      void Clear();

    protected:
      class CLuaManager *_mgr;       // Managing CLuaManager instance
      string             _nameID;    // Name of this CLuaExecutable instance
      vector<char>       _byteCodes; // The executable byte codes

      // Friend classes
      friend class CLuaManager;
      friend class CLuaExecutionSpace;
  };

} // namespace vge
#endif
