// ############################################################################################
//  mcatedit-MCatEdit.h
//  Definition of mcatedit's CMCatEdit and its dialog page classes
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################
#ifndef _UTIL___MCATEDIT_MCATEDIT_H_
#define _UTIL___MCATEDIT_MCATEDIT_H_

// mcatedit's headers
#include "mcatedit-MCBuilder.h"


// Just for shortcut
typedef CDialogApplication<> CMCatEditBase;

// Foward reference of the application class
class CMCatEdit;

// Foward reference of the dialog main page
class CPage_Main;


//
// Our dialog page: Directory Selection
//
class CPage_SelDir : public CDialogPage {
  public:
    typedef void(CPage_Main::*PFNToInvoke)(const string &dirName);

  public:
    CPage_SelDir(CPage_Main &mainPage, const string &startDir);
    virtual ~CPage_SelDir() throw();

    // Management functions
    void SetParam(PFNToInvoke pfnTI, const string &infoText);

    // Controls' event handlers
    void _onLstSDir(_CControl *ctrl, bool dblClick);
    void _onBtnOK(_CControl *ctrl);
    void _onBtnCancel(_CControl *ctrl);

    // Reimplemented from CDialogPage
    virtual void InitializeAllControls();

    // Reimplemented from CDialogPage
    virtual void HandleEvent_OnKeyPress(EKeySymbol keySymbol, const char *str);

    // Make the dialog main page class as friend class
    friend class CPage_Main;

  protected:
    // Helper functions
    const string _listDir(const char *path = NULL);

    // Main data
    CMCatEdit     &_app;
    CPage_Main    &_mPage;
    CVideoStorage  _vStrg;
    PFNToInvoke    _pfnTI;
    string         _startDir;
    // Controls
    CPanelBar      pbrSDir;
    CHLayout       hlySDir;
    CLabel         lblSDirInfo;
    CLabel         lblSDirPath;
    CListBox       lstSDir;
    CVLayout       vlySDir;
    CButton        btnSDirOK;
    CButton        btnSDirCancel;
};


//
// Our dialog page: Locale Management
//
class CPage_MLoc : public CDialogPage {
  public:
    CPage_MLoc(CPage_Main &mainPage);
    virtual ~CPage_MLoc() throw();

    // Controls' event handlers
    void _onBtnNew(_CControl *ctrl);
    void _onBtnDel(_CControl *ctrl);
    void _onLstLoc(_CControl *ctrl, bool dblClick);
    void _onBtnDone(_CControl *ctrl);

    // Reimplemented from CDialogPage
    virtual void InitializeAllControls();

    // Reimplemented from CDialogPage
    virtual void HandleEvent_OnKeyPress(EKeySymbol keySymbol, const char *str);

    // Make the dialog main page class as friend class
    friend class CPage_Main;

  protected:
    // Callback functions
#ifndef __USE_UTF8_DIALOG_STRING__
    static bool _sltbCallback(ETBEvent eventCode, CVideoEnv &instVideoEnv,
                              CSLTextBox &slTextBox, string &editBuffer,
                              string *pasteBuffer, char &charCode,
                              EKeySymbol keySym);
#else
    static bool _sltbCallback(ETBEvent eventCode, CVideoEnv &instVideoEnv,
                              CSLTextBox &slTextBox, CUString &editBuffer,
                              CUString *pasteBuffer, CUString::Utf32 &charCode,
                              EKeySymbol keySym);
#endif

    // Main data
    CMCatEdit     &_app;
    CPage_Main    &_mPage;
    CVideoStorage  _vStrg;
    // Controls
    CPanelBar      pbrLoc;
    CHLayout       hlyLoc;
    CVLayout       vlyLoc;
    CButton        btnLocAdd;
    CButton        btnLocDel;
    CListBox       lstLoc;
    CButton        btnLocDone;
};


//
// Our dialog page: Find in ID or MT
//
class CPage_Find : public CDialogPage {
  public:
    CPage_Find(CPage_Main &mainPage);
    virtual ~CPage_Find() throw();

    // Management functions
    void SetParam(bool findID);
    void SetParam(uint32_t cur, uint32_t tot, const string &text);

    // Controls' event handlers
    void _onBtnSearch(_CControl *ctrl);
    void _onBtnFirst(_CControl *ctrl);
    void _onBtnPrev(_CControl *ctrl);
    void _onBtnNext(_CControl *ctrl);
    void _onBtnLast(_CControl *ctrl);
    void _onBtnDone(_CControl *ctrl);

    // Reimplemented from CDialogPage
    virtual void InitializeAllControls();

    // Reimplemented from CDialogPage
    virtual void HandleEvent_OnKeyPress(EKeySymbol keySymbol, const char *str);

    // Make the dialog main page class as friend class
    friend class CPage_Main;

  protected:
    // Helper functions
    void _updateSearchDisplay();

    // Main data
    string         RS_TITLE;
    CMCatEdit     &_app;
    CPage_Main    &_mPage;
    CVideoStorage  _vStrg;
    // Controls: Main section
    CHLayout       hlyFind;
    CPanelBar      pbrFindOpt;
    CPanel         pnlFindRes;
    // Controls: Setting section
    CHLayout          hlyFindOpt;
    CVHLayoutCompound vhgFindOpt;
    CLabel            lblFindOptFI;
    CRadioButton      rdoFindOptID;
    CRadioButton      rdoFindOptMT;
    CLabel            lblFindOptMF;
    CRadioButton      rdoFindOptSingle;
    CRadioButton      rdoFindOptAll;
    CRadioButton      rdoFindOptExact;
    CCheckBox         chkFindOptCS;
    CLabel            lblFindOptTF;
    CVLayout          vlyFindOpt;
    CSLTextBox        txtFindOpt;
    CButton           btnFindOptSearch;
    // Controls: Result section
    CHLayout          hlyFindRes;
    CLabel            lblFindResR;
    CMLTextBox        txtFindRes;
    CVLayout          vlyFindRes;
    CButton           btnFindResFirst;
    CButton           btnFindResPrev;
    CButton           btnFindResNext;
    CButton           btnFindResLast;
    CButton           btnFindResDone;
};


//
// Our dialog page: Main Page
//
class CPage_Main : public CDialogPage {
  public:
    CPage_Main(CMCatEdit &appClass);
    virtual ~CPage_Main() throw();

    // Information functions
    inline CMCatEdit &GetAppInst()
    { return(_app); }

    // Management functions
    void UpdateAllControlsEnableStates();

    // Controls' event handlers: Menu bar section
    void _onMnuFileNew(_CControl *ctrl);
    void _onMnuFileNew(const string &dirName);
    void _onMnuFileOpen(_CControl *ctrl);
    void _onMnuFileOpen(const string &dirName);
    void _onMnuFileSave(_CControl *ctrl);
    void _onMnuFileSaveAs(_CControl *ctrl);
    void _onMnuFileSaveAs(const string &dirName);
    void _onMnuFileIndexInfo(_CControl *ctrl);
    void _onMnuFileLocale(_CControl *ctrl);
    void _onMnuFileAbout(_CControl *ctrl);
    // Controls' event handlers: Message ID section
    void _onBtnIDAdd(_CControl *ctrl);
    void _onBtnIDDel(_CControl *ctrl);
    void _onBtnIDFirst(_CControl *ctrl);
    void _onBtnIDPrev(_CControl *ctrl);
    void _onBtnIDNext(_CControl *ctrl);
    void _onBtnIDLast(_CControl *ctrl);
    void _onBtnIDFind(_CControl *ctrl);
    // Controls' event handlers: Message translation section
    void _onBtnMTFirst(_CControl *ctrl);
    void _onBtnMTPrev(_CControl *ctrl);
    void _onBtnMTNext(_CControl *ctrl);
    void _onBtnMTLast(_CControl *ctrl);
    void _onBtnMTFind(_CControl *ctrl);

    // Reimplemented from CDialogPage
    virtual void HandleEvent_OnKeyPress(EKeySymbol keySymbol, const char *str);

    // Reimplemented from CDialogPage
    virtual void InitializeAllControls();

    // Make the dialog sub-pages classes as friend classes
    friend class CPage_SelDir;
    friend class CPage_MLoc;
    friend class CPage_Find;

  protected:
    // Reimplemented from CDialogPage
    virtual void DisplayTooltip(const char *tooltip);

    // Helper functions
    void _saveCurValues();
    bool _searchUntranslatedMsg(size_t startIndex, size_t endIndex);

    // Main data
    string         ID_TITLE;
    string         MT_TITLE;
    CMCatEdit     &_app;
    CVideoStorage  _vStrg;
    CMCBuilder     _theCatalog;
    // Controls: Main section
    CHLayout       hlyMain;
    CVLayout       vlyMenu;
    CPanel         pnlID;
    CPanel         pnlMT;
    CStatusBar     stbSB;
    // Controls: Menu bar section
    CButton        btnMNNew;
    CButton        btnMNOpen;
    CButton        btnMNSave;
    CButton        btnMNSaveAs;
    CButton        btnMNIndexInfo;
    CButton        btnMNLocale;
    CButton        btnMNAbout;
    // Controls: Message ID section
    CHLayout       hlyID;
    CLabel         lblID;
    CMLTextBox     txtID;
    CVLayout       vlyID;
    CButton        btnIDAdd;
    CButton        btnIDDel;
    CButton        btnIDFirst;
    CButton        btnIDPrev;
    CButton        btnIDNext;
    CButton        btnIDLast;
    CButton        btnIDFind;
    // Controls: Message translation section
    CHLayout       hlyMT;
    CLabel         lblMT;
    CMLTextBox     txtMT;
    CVLayout       vlyMT;
    CButton        btnMTFirst;
    CButton        btnMTPrev;
    CButton        btnMTNext;
    CButton        btnMTLast;
    CButton        btnMTFind;
};


//
// Our application class
//
class CMCatEdit : public CMCatEditBase {
  public:
    enum Mode { ModeNormal, ModeNew, ModeOpen };

    // Constants for versioning
    static const string TEXT_VERSION;
    static const string TEXT_YEAR;

    // Constants for page name
    static const string PAGE_MAIN; // Main page
    static const string PAGE_SDIR; // Directory selection page
    static const string PAGE_MLOC; // Manage locale page
    static const string PAGE_FIND; // Find in ID or MT page

  public:
    CMCatEdit(uint32_t wWidth, uint32_t wHeight, const string &startDir, Mode mode);
    virtual ~CMCatEdit() throw();

    // Management functions
    void SetWindowTitle(const string &addText);
    void SelectPage(const string &pageName);

    // Standard event handlers
    virtual void EventUpdateFrame(bool bWindowIsActive);
    virtual void EventWindowActivated();
    virtual void EventWindowDeactivated();
    virtual void EventCloseWindow();

    // Make the dialog pages classes as friend classes
    friend class CPage_SelDir;
    friend class CPage_MLoc;
    friend class CPage_Find;
    friend class CPage_Main;

  private:
    // Constants for video storage
    static const uint32_t APP_INACTIVE_STORE = 0;
    static const uint32_t PRE_DIALOG_STORE   = 1;
    static const uint32_t VSTORAGE_CAPACITY  = 2;

    // Common data
    string        WINDOW_TITLE;
    EKeySymbol    eKey;
    CVideoStorage csVStrg;
    // Page data
    CPage_Main    pageMain;
    CPage_SelDir  pageSDir;
    CPage_MLoc    pageMLoc;
    CPage_Find    pageFind;
};

#endif
