// ############################################################################################
//  roomedit-RoomEdit-Main.cc
//  Implementation of roomedit's CRoomEdit class (common functions)
//  -------------------------------------------------------------------------------------------
//  Copyright (C) 2004-2006 Aloysius Indrayanto
//  This file is part of VortexGE
//
//  This program is free software; you can redistribute it and/or modify it under the terms of
//  the GNU Lesser General Public License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License along with this
//  library; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
//  Boston, MA 02111-1307 USA.
// ############################################################################################

// roomedit's headers
#include "roomedit-RoomEdit.h"
#include "roomedit-WIcon.h"


//
// RoomEdit implementation
//
const string CRoomEdit::TEXT_VERSION = "v0.3.0";
const string CRoomEdit::TEXT_YEAR    = "2004-2006";

const string CRoomEdit::PAGE_MAIN = "Main";
const string CRoomEdit::PAGE_HELP = "Help";
const string CRoomEdit::PAGE_GSZ  = "GSZ";
const string CRoomEdit::PAGE_NPS  = "NPS";

const uint32_t CRoomEdit::APP_INACTIVE_STORE;
const uint32_t CRoomEdit::PRE_DIALOG_STORE;
const uint32_t CRoomEdit::VSTORAGE_CAPACITY;


CRoomEdit::CRoomEdit(uint32_t wWidth, uint32_t wHeight, bool lowRes3D, Mode mode, const string &file)
: CRoomEditBase("", NULL, wWidth, wHeight, 0, 0, EAIM_Disabled, E3DIM_Manual),
  pageMain     (*this, lowRes3D),
  pageHelp     (pageMain),
  pageGSZ      (pageMain),
  pageNPS      (pageMain)
{
  // (Re-)initialize the window with our specific settings
  WINDOW_TITLE = _M("VortexGE Room Editor");
  SetWindowTitle("");
  iVideoEnv().SetWindowIcon(roomedit_xpm);
  iVideoEnv().MoveCenterWindow(true);
  //iVideoEnv().SetFullcreen(true);

  // Register the pages (remember that the 1st page will become the main page)
  _dialogPageManager.RegisterPage(pageMain, PAGE_MAIN);
  _dialogPageManager.RegisterPage(pageHelp, PAGE_HELP);
  _dialogPageManager.RegisterPage(pageGSZ,  PAGE_GSZ);
  _dialogPageManager.RegisterPage(pageNPS,  PAGE_NPS);

  // Initialize video storage
  csVStrg.Initialize(&iVideoEnv(), VSTORAGE_CAPACITY);

  // Initialize dialog
  pageHelp.InitializeAllControls();
  pageGSZ.InitializeAllControls();
  pageNPS.InitializeAllControls();
  pageMain.InitializeAllControls();
  pageMain.ResetAllControls();
  pageMain.UpdateAllControlsEnableStates();
  pageMain.UpdateInfoText();

  // Draw the main page for the first time
  _dialogPageManager.RedrawPage();

  // Do we ordered to create a new catalog in the start directory ?
  if(mode == ModeNew) {
  }

  // Do we ordered to create open an existing catalogin the start directory ?
  if(mode == ModeOpen) {
  }
}

CRoomEdit::~CRoomEdit() throw()
{
}

void CRoomEdit::SetWindowTitle(const string &addText)
{
  if(addText.empty()) {
    iVideoEnv().SetWindowCaption(WINDOW_TITLE.c_str());
    return;
  }

  const string tmp = WINDOW_TITLE + " - " + addText;
  iVideoEnv().SetWindowCaption(tmp.c_str());
}

void CRoomEdit::SelectPage(const string &pageName)
{
  _dialogPageManager.SelectPage(pageName);
  _dialogPageManager.RedrawPage();
  iVideoEnv().UpdateDisplay();
}
